@@ -24,7 +24,7 @@ class ReferenceValueMap(ABC):
     """Maps from reference index to reference value of ReferencedData."""
 
     def __init__(self, color_map: dict[int, str] = None):
-        self._map = dict() if color_map is None else color_map
+        self._map = {} if color_map is None else color_map
 
     def __getitem__(self, item: int) -> str:
         return self._map[item]
