@@ -1,4 +1,4 @@
-#  Copyright (c) 2020 Mira Geoscience Ltd.
+#  Copyright (c) 2021 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -40,12 +40,6 @@ class Label(ObjectBase):
 
         super().__init__(object_type, **kwargs)
 
-        if object_type.name == "None":
-            self.entity_type.name = "Label"
-
-        # if object_type.description is None:
-        #     self.entity_type.description = "Label"
-
         object_type.workspace._register_object(self)
 
     @classmethod
