@@ -1,4 +1,4 @@
-#  Copyright (c) 2020 Mira Geoscience Ltd.
+#  Copyright (c) 2021 Mira Geoscience Ltd.
 #
 #  This file is part of geoh5py.
 #
@@ -33,9 +33,6 @@ class NoTypeObject(ObjectBase):
     def __init__(self, object_type: ObjectType, **kwargs):
         super().__init__(object_type, **kwargs)
 
-        if object_type.name == "None":
-            self.entity_type.name = "Unknown"
-
         object_type.workspace._register_object(self)
 
     @classmethod
