@@ -16,10 +16,12 @@
 #  along with geoh5py.  If not, see <https://www.gnu.org/licenses/>.
 
 
+from __future__ import annotations
+
 import uuid
 from abc import abstractmethod
 from datetime import datetime
-from typing import TYPE_CHECKING, List, Optional, Union
+from typing import TYPE_CHECKING
 
 import numpy as np
 
@@ -47,7 +49,7 @@ class ObjectBase(Entity):
     def __init__(self, object_type: ObjectType, **kwargs):
         assert object_type is not None
         self._entity_type = object_type
-        self._property_groups: List[PropertyGroup] = []
+        self._property_groups: list[PropertyGroup] = []
         self._last_focus = "None"
         self._comments = None
         # self._clipping_ids: List[uuid.UUID] = []
@@ -88,9 +90,7 @@ class ObjectBase(Entity):
         else:
             self.comments.values = self.comments.values + [comment_dict]
 
-    def add_data(
-        self, data: dict, property_group: str = None
-    ) -> Union[Data, List[Data]]:
+    def add_data(self, data: dict, property_group: str = None) -> Data | list[Data]:
         """
         Create :obj:`~geoh5py.data.data.Data` from dictionary of name and arguments.
         The provided arguments can be any property of the target Data class.
@@ -118,16 +118,9 @@ class ObjectBase(Entity):
                 f"Given value to data {name} should of type {dict}. "
                 f"Type {type(attr)} given instead."
             )
-            assert "values" in list(
-                attr.keys()
-            ), f"Given attr for data {name} should include 'values'"
-
             attr["name"] = name
-
             self.validate_data_association(attr)
             entity_type = self.validate_data_type(attr)
-
-            # Re-order to set parent first
             kwargs = {"parent": self, "association": attr["association"]}
             for key, val in attr.items():
                 if key in ["parent", "association", "entity_type", "type"]:
@@ -143,13 +136,15 @@ class ObjectBase(Entity):
 
             data_objects.append(data_object)
 
+        self.workspace.finalize()
+
         if len(data_objects) == 1:
             return data_object
 
         return data_objects
 
     def add_data_to_group(
-        self, data: Union[List, Data, uuid.UUID, str], name: str
+        self, data: list | Data | uuid.UUID | str, name: str
     ) -> PropertyGroup:
         """
         Append data children to a :obj:`~geoh5py.groups.property_group.PropertyGroup`
@@ -182,7 +177,7 @@ class ObjectBase(Entity):
         return prop_group
 
     def remove_data_from_group(
-        self, data: Union[List, Data, uuid.UUID, str], name: str = None
+        self, data: list | Data | uuid.UUID | str, name: str = None
     ):
         """
         Remove data children to a :obj:`~geoh5py.groups.property_group.PropertyGroup`
@@ -256,7 +251,7 @@ class ObjectBase(Entity):
 
     @classmethod
     def find_or_create_type(
-        cls, workspace: "workspace.Workspace", **kwargs
+        cls, workspace: workspace.Workspace, **kwargs
     ) -> ObjectType:
         """
         Find or create a type instance for a given object class.
@@ -277,7 +272,7 @@ class ObjectBase(Entity):
 
         :return: A new or existing :obj:`~geoh5py.groups.property_group.PropertyGroup`
         """
-        if "name" in list(kwargs.keys()) and any(
+        if "name" in kwargs and any(
             pg.name == kwargs["name"] for pg in self.property_groups
         ):
             prop_group = [
@@ -289,7 +284,7 @@ class ObjectBase(Entity):
 
         return prop_group
 
-    def get_data(self, name: str) -> List[Data]:
+    def get_data(self, name: str) -> list[Data]:
         """
         Get a child :obj:`~geoh5py.data.data.Data` by name.
 
@@ -305,7 +300,7 @@ class ObjectBase(Entity):
 
         return entity_list
 
-    def get_data_list(self) -> List[str]:
+    def get_data_list(self) -> list[str]:
         """
         Get a list of names of all children :obj:`~geoh5py.data.data.Data`.
 
@@ -329,7 +324,7 @@ class ObjectBase(Entity):
         self._last_focus = value
 
     @property
-    def n_cells(self) -> Optional[int]:
+    def n_cells(self) -> int | None:
         """
         :obj:`int`: Number of cells.
         """
@@ -338,7 +333,7 @@ class ObjectBase(Entity):
         return None
 
     @property
-    def n_vertices(self) -> Optional[int]:
+    def n_vertices(self) -> int | None:
         """
         :obj:`int`: Number of vertices.
         """
@@ -347,14 +342,14 @@ class ObjectBase(Entity):
         return None
 
     @property
-    def property_groups(self) -> List[PropertyGroup]:
+    def property_groups(self) -> list[PropertyGroup]:
         """
         :obj:`list` of :obj:`~geoh5py.groups.property_group.PropertyGroup`.
         """
         return self._property_groups
 
     @property_groups.setter
-    def property_groups(self, prop_groups: List[PropertyGroup]):
+    def property_groups(self, prop_groups: list[PropertyGroup]):
         # Check for existing property_group
         for prop_group in prop_groups:
             if not any(
@@ -378,7 +373,7 @@ class ObjectBase(Entity):
         Get a dictionary of attributes and validate the data 'association' keyword.
         """
 
-        if "association" in list(attribute_dict.keys()):
+        if "association" in attribute_dict.keys():
             assert attribute_dict["association"] in [
                 enum.name for enum in DataAssociationEnum
             ], (
@@ -404,28 +399,30 @@ class ObjectBase(Entity):
         """
         Get a dictionary of attributes and validate the type of data.
         """
-
-        if "entity_type" in list(attribute_dict.keys()):
-            entity_type = attribute_dict["entity_type"]
-        elif "type" in list(attribute_dict.keys()):
-            assert attribute_dict["type"].upper() in list(
-                PrimitiveTypeEnum.__members__.keys()
-            ), f"Data 'type' should be one of {list(PrimitiveTypeEnum.__members__.keys())}"
-            entity_type = {"primitive_type": attribute_dict["type"].upper()}
-        else:
-            if isinstance(attribute_dict["values"], np.ndarray) and (
-                attribute_dict["values"].dtype in [np.float32, np.float64]
-            ):
-                entity_type = {"primitive_type": "FLOAT"}
-            elif isinstance(attribute_dict["values"], np.ndarray) and (
-                attribute_dict["values"].dtype in [np.uint32, np.int32]
-            ):
-                entity_type = {"primitive_type": "INTEGER"}
-            elif isinstance(attribute_dict["values"], str):
-                entity_type = {"primitive_type": "TEXT"}
+        entity_type = attribute_dict.get("entity_type")
+        if entity_type is None:
+            primitive_type = attribute_dict.get("type")
+            if primitive_type is not None:
+                assert primitive_type.upper() in list(
+                    PrimitiveTypeEnum.__members__.keys()
+                ), f"Data 'type' should be one of {list(PrimitiveTypeEnum.__members__.keys())}"
+                entity_type = {"primitive_type": primitive_type.upper()}
             else:
-                raise NotImplementedError(
-                    "Only add_data values of type FLOAT, INTEGER and TEXT have been implemented"
-                )
+                values = attribute_dict.get("values")
+                if values is None or (
+                    isinstance(values, np.ndarray)
+                    and (values.dtype in [np.float32, np.float64])
+                ):
+                    entity_type = {"primitive_type": "FLOAT"}
+                elif isinstance(values, np.ndarray) and (
+                    values.dtype in [np.uint32, np.int32]
+                ):
+                    entity_type = {"primitive_type": "INTEGER"}
+                elif isinstance(values, str):
+                    entity_type = {"primitive_type": "TEXT"}
+                else:
+                    raise NotImplementedError(
+                        "Only add_data values of type FLOAT, INTEGER and TEXT have been implemented"
+                    )
 
         return entity_type
